@@ -1,24 +1,37 @@
 from __future__ import annotations
 
 from spacepackets.log import get_console_logger
-from spacepackets.cfdp.definitions import LenInBytes, FileSize, PduType, \
-    SegmentMetadataFlag, CrcFlag, TransmissionModes, Direction, SegmentationControl
-from spacepackets.cfdp.conf import PduConfig, get_default_pdu_crc_mode, get_default_file_size, \
-    get_entity_ids
+from spacepackets.cfdp.definitions import (
+    LenInBytes,
+    FileSize,
+    PduType,
+    SegmentMetadataFlag,
+    CrcFlag,
+    TransmissionModes,
+    Direction,
+    SegmentationControl,
+)
+from spacepackets.cfdp.conf import (
+    PduConfig,
+    get_default_pdu_crc_mode,
+    get_default_file_size,
+    get_entity_ids,
+)
 
 
 class PduHeader:
     """This class encapsulates the fixed-format PDU header.
     For more, information, refer to CCSDS 727.0-B-5 p.75"""
+
     VERSION_BITS = 0b0010_0000
     FIXED_LENGTH = 4
 
     def __init__(
-            self,
-            pdu_type: PduType,
-            segment_metadata_flag: SegmentMetadataFlag,
-            pdu_data_field_len: int,
-            pdu_conf: PduConfig
+        self,
+        pdu_type: PduType,
+        segment_metadata_flag: SegmentMetadataFlag,
+        pdu_data_field_len: int,
+        pdu_conf: PduConfig,
     ):
         """Constructor for PDU header
 
@@ -34,7 +47,8 @@ class PduHeader:
 
         self.len_entity_id = 0
         self.set_entity_ids(
-            source_entity_id=pdu_conf.source_entity_id, dest_entity_id=pdu_conf.dest_entity_id
+            source_entity_id=pdu_conf.source_entity_id,
+            dest_entity_id=pdu_conf.dest_entity_id,
         )
 
         self.len_transaction_seq_num = 0
@@ -60,8 +74,8 @@ class PduHeader:
             if source_entity_id == bytes() or dest_entity_id == bytes():
                 logger = get_console_logger()
                 logger.warning(
-                    'Can not set default value for source entity ID or destination entity ID '
-                    'because it has not been set yet'
+                    "Can not set default value for source entity ID or destination entity ID "
+                    "because it has not been set yet"
                 )
                 raise ValueError
         self.pdu_conf.source_entity_id = source_entity_id
@@ -71,11 +85,11 @@ class PduHeader:
             dest_id_check = self.check_len_in_bytes(len(dest_entity_id))
         except ValueError:
             logger = get_console_logger()
-            logger.warning('Invalid length of entity IDs passed')
+            logger.warning("Invalid length of entity IDs passed")
             raise ValueError
         if dest_id_check != self.len_entity_id:
             logger = get_console_logger()
-            logger.warning('Length of destination ID and source ID are not the same')
+            logger.warning("Length of destination ID and source ID are not the same")
             raise ValueError
 
     @property
@@ -86,10 +100,12 @@ class PduHeader:
     def transaction_seq_num(self, transaction_seq_num: bytes):
         if transaction_seq_num is not None:
             try:
-                self.len_transaction_seq_num = self.check_len_in_bytes(len(transaction_seq_num))
+                self.len_transaction_seq_num = self.check_len_in_bytes(
+                    len(transaction_seq_num)
+                )
             except ValueError:
                 logger = get_console_logger()
-                logger.warning('Invalid length of transaction sequence number passed')
+                logger.warning("Invalid length of transaction sequence number passed")
                 raise ValueError
         self.pdu_conf.transaction_seq_num = transaction_seq_num
 
@@ -171,15 +187,20 @@ class PduHeader:
     def pack(self) -> bytearray:
         header = bytearray()
         header.append(
-            self.VERSION_BITS | (self.pdu_type << 4) | (self.pdu_conf.direction << 3) |
-            (self.pdu_conf.trans_mode << 2) | (self.pdu_conf.crc_flag << 1) |
-            self.pdu_conf.file_size
+            self.VERSION_BITS
+            | (self.pdu_type << 4)
+            | (self.pdu_conf.direction << 3)
+            | (self.pdu_conf.trans_mode << 2)
+            | (self.pdu_conf.crc_flag << 1)
+            | self.pdu_conf.file_size
         )
-        header.append((self.pdu_data_field_len >> 8) & 0xff)
-        header.append(self.pdu_data_field_len & 0xff)
+        header.append((self.pdu_data_field_len >> 8) & 0xFF)
+        header.append(self.pdu_data_field_len & 0xFF)
         header.append(
-            self.pdu_conf.seg_ctrl << 7 | self.len_entity_id << 4 |
-            self.segment_metadata_flag << 3 | self.len_transaction_seq_num
+            self.pdu_conf.seg_ctrl << 7
+            | self.len_entity_id << 4
+            | self.segment_metadata_flag << 3
+            | self.len_transaction_seq_num
         )
         header.extend(self.pdu_conf.source_entity_id)
         header.extend(self.pdu_conf.transaction_seq_num)
@@ -193,7 +214,7 @@ class PduHeader:
             pdu_type=PduType.FILE_DIRECTIVE,
             segment_metadata_flag=SegmentMetadataFlag.NOT_PRESENT,
             pdu_data_field_len=0,
-            pdu_conf=empty_conf
+            pdu_conf=empty_conf,
         )
 
     @classmethod
@@ -206,7 +227,7 @@ class PduHeader:
         """
         if len(raw_packet) < cls.FIXED_LENGTH:
             logger = get_console_logger()
-            logger.warning('Can not unpack less than four bytes into PDU header')
+            logger.warning("Can not unpack less than four bytes into PDU header")
             raise ValueError
         pdu_header = cls.__empty()
         pdu_header.pdu_type = (raw_packet[0] & 0x10) >> 4
@@ -215,25 +236,38 @@ class PduHeader:
         pdu_header.crc_flag = (raw_packet[0] & 0x02) >> 1
         pdu_header.file_size = raw_packet[0] & 0x01
         pdu_header.pdu_data_field_len = raw_packet[1] << 8 | raw_packet[2]
-        pdu_header.segmentation_control = SegmentationControl((raw_packet[3] & 0x80) >> 7)
+        pdu_header.segmentation_control = SegmentationControl(
+            (raw_packet[3] & 0x80) >> 7
+        )
         pdu_header.len_entity_id = cls.check_len_in_bytes((raw_packet[3] & 0x70) >> 4)
-        pdu_header.segment_metadata_flag = SegmentMetadataFlag((raw_packet[3] & 0x08) >> 3)
-        pdu_header.len_transaction_seq_num = cls.check_len_in_bytes(raw_packet[3] & 0x07)
-        expected_remaining_len = 2 * pdu_header.len_entity_id + pdu_header.len_transaction_seq_num
+        pdu_header.segment_metadata_flag = SegmentMetadataFlag(
+            (raw_packet[3] & 0x08) >> 3
+        )
+        pdu_header.len_transaction_seq_num = cls.check_len_in_bytes(
+            raw_packet[3] & 0x07
+        )
+        expected_remaining_len = (
+            2 * pdu_header.len_entity_id + pdu_header.len_transaction_seq_num
+        )
         if len(raw_packet) - cls.FIXED_LENGTH < expected_remaining_len:
             logger = get_console_logger()
-            logger.warning('Raw packet too small for PDU header')
+            logger.warning("Raw packet too small for PDU header")
             raise ValueError
         current_idx = 4
-        source_entity_id = \
-            raw_packet[current_idx: current_idx + pdu_header.len_entity_id]
+        source_entity_id = raw_packet[
+            current_idx : current_idx + pdu_header.len_entity_id
+        ]
         current_idx += pdu_header.len_entity_id
-        pdu_header.transaction_seq_num = \
-            raw_packet[current_idx: current_idx + pdu_header.len_transaction_seq_num]
+        pdu_header.transaction_seq_num = raw_packet[
+            current_idx : current_idx + pdu_header.len_transaction_seq_num
+        ]
         current_idx += pdu_header.len_transaction_seq_num
-        dest_entity_id = \
-            raw_packet[current_idx: current_idx + pdu_header.len_entity_id]
-        pdu_header.set_entity_ids(source_entity_id=source_entity_id, dest_entity_id=dest_entity_id)
+        dest_entity_id = raw_packet[
+            current_idx : current_idx + pdu_header.len_entity_id
+        ]
+        pdu_header.set_entity_ids(
+            source_entity_id=source_entity_id, dest_entity_id=dest_entity_id
+        )
         return pdu_header
 
     @staticmethod
@@ -243,8 +277,8 @@ class PduHeader:
         except ValueError:
             logger = get_console_logger()
             logger.warning(
-                'Unsupported length field detected. '
-                'Only 1, 2, 4 and 8 bytes are supported'
+                "Unsupported length field detected. "
+                "Only 1, 2, 4 and 8 bytes are supported"
             )
             raise ValueError
         return len_in_bytes
@@ -252,6 +286,7 @@ class PduHeader:
 
 class HasPduHeader:
     """Encapsulate common functions for classes which have a PDU header"""
+
     def __init__(self, pdu_header: PduHeader):
         self.pdu_header = pdu_header
 
