@@ -1,5 +1,8 @@
 from __future__ import annotations
 
+import struct
+from typing import Optional
+
 from crcmod.predefined import mkPredefinedCrcFun
 
 from spacepackets.util import PrintFormats, get_printable_data_string
@@ -9,9 +12,15 @@ from spacepackets.ccsds.spacepacket import (
     SPACE_PACKET_HEADER_SIZE,
     get_total_space_packet_len_from_len_field,
     PacketTypes,
+    SpacePacket,
 )
 from spacepackets.ccsds.time import CdsShortTimestamp, read_p_field
-from spacepackets.ecss.conf import get_pus_tm_version, PusVersion, get_default_tm_apid
+from spacepackets.ecss.conf import (
+    get_pus_tm_version,
+    PusVersion,
+    get_default_tm_apid,
+    FETCH_GLOBAL_APID,
+)
 
 
 def get_service_from_raw_pus_packet(raw_bytearray: bytearray) -> int:
@@ -26,6 +35,179 @@ def get_service_from_raw_pus_packet(raw_bytearray: bytearray) -> int:
     return raw_bytearray[7]
 
 
+class PusTmSecondaryHeader:
+    """Unpacks the PUS telemetry packet secondary header.
+    Currently only supports CDS short timestamps"""
+
+    HEADER_SIZE_WITHOUT_TIME_PUS_A = 4
+    HEADER_SIZE_WITHOUT_TIME_PUS_C = 7
+
+    def __init__(
+        self,
+        service: int,
+        subservice: int,
+        time: CdsShortTimestamp,
+        message_counter: int,
+        dest_id: int = 0,
+        spacecraft_time_ref: int = 0,
+        pus_version: PusVersion = PusVersion.GLOBAL_CONFIG,
+    ):
+        """Create a PUS telemetry secondary header object.
+
+        :param pus_version: PUS version. ESA PUS is not supported
+        :param service:
+        :param subservice:
+        :param time: Time field
+        :param message_counter: 8 bit counter for PUS A, 16 bit counter for PUS C
+        :param dest_id: Destination ID if PUS C is used
+        :param spacecraft_time_ref: Space time reference if PUS C is used
+        """
+        self.pus_version = pus_version
+        if self.pus_version == PusVersion.GLOBAL_CONFIG:
+            self.pus_version = get_pus_tm_version()
+        if (
+            self.pus_version != PusVersion.PUS_A
+            and self.pus_version != PusVersion.PUS_C
+        ):
+            raise ValueError
+        self.spacecraft_time_ref = spacecraft_time_ref
+        if service > pow(2, 8) - 1 or service < 0:
+            raise ValueError(f"Invalid Service {service}")
+        if subservice > pow(2, 8) - 1 or subservice < 0:
+            raise ValueError(f"Invalid Subservice {subservice}")
+        self.service = service
+        self.subservice = subservice
+        if (
+            self.pus_version == PusVersion.PUS_A and message_counter > pow(2, 8) - 1
+        ) or (
+            self.pus_version == PusVersion.PUS_C and message_counter > pow(2, 16) - 1
+        ):
+            raise ValueError
+        self.message_counter = message_counter
+        self.dest_id = dest_id
+        self.time = time
+
+    @classmethod
+    def __empty(cls) -> PusTmSecondaryHeader:
+        return PusTmSecondaryHeader(
+            pus_version=PusVersion.PUS_C,
+            service=0,
+            subservice=0,
+            time=CdsShortTimestamp.init_from_current_time(),
+            message_counter=0,
+        )
+
+    def pack(self) -> bytearray:
+        secondary_header = bytearray()
+        if self.pus_version == PusVersion.PUS_A:
+            secondary_header.append((self.pus_version & 0x07) << 4)
+        elif self.pus_version == PusVersion.PUS_C:
+            secondary_header.append(self.pus_version << 4 | self.spacecraft_time_ref)
+        secondary_header.append(self.service)
+        secondary_header.append(self.subservice)
+        if self.pus_version == PusVersion.PUS_A:
+            secondary_header.append(self.message_counter)
+        elif self.pus_version == PusVersion.PUS_C:
+            secondary_header.append((self.message_counter & 0xFF00) >> 8)
+            secondary_header.append(self.message_counter & 0xFF)
+            secondary_header.append((self.dest_id & 0xFF00) >> 8)
+            secondary_header.append(self.dest_id & 0xFF)
+        secondary_header.extend(self.time.pack())
+        return secondary_header
+
+    @classmethod
+    def unpack(
+        cls, header_start: bytes, pus_version: PusVersion
+    ) -> PusTmSecondaryHeader:
+        """Unpack the PUS TM secondary header from the raw packet starting at the header index.
+        The user still needs to specify the PUS version because the version field is parsed
+        differently depending on the PUS version.
+
+        :param header_start:
+        :param pus_version:
+        :raises ValueError: bytearray too short or PUS version missmatch.
+        :return:
+        """
+        if len(header_start) < cls.HEADER_SIZE_WITHOUT_TIME_PUS_A:
+            logger = get_console_logger()
+            logger.warning("Passed bytearray too short")
+            raise ValueError
+        if pus_version == PusVersion.GLOBAL_CONFIG:
+            pus_version = get_pus_tm_version()
+        secondary_header = cls.__empty()
+        current_idx = 0
+        if pus_version == PusVersion.PUS_A:
+            secondary_header.pus_version = (header_start[current_idx] & 0x70) >> 4
+            if secondary_header.pus_version != PusVersion.PUS_A:
+                logger = get_console_logger()
+                logger.warning(
+                    f"PUS version field value {secondary_header.pus_version} "
+                    f"found where PUS A {pus_version} was expected!"
+                )
+                raise ValueError
+        elif pus_version == PusVersion.PUS_C:
+            secondary_header.pus_version = (header_start[current_idx] & 0xF0) >> 4
+            if secondary_header.pus_version != PusVersion.PUS_C:
+                logger = get_console_logger()
+                logger.warning(
+                    f"PUS version field value {secondary_header.pus_version} "
+                    f"found where PUS C {pus_version} was expected!"
+                )
+                raise ValueError
+            secondary_header.spacecraft_time_ref = header_start[current_idx] & 0x0F
+        if len(header_start) < secondary_header.header_size:
+            logger = get_console_logger()
+            logger.warning(
+                f"Invalid PUS data field header size, "
+                f"less than expected {secondary_header.header_size} bytes"
+            )
+            raise ValueError
+        current_idx += 1
+        secondary_header.service = header_start[current_idx]
+        current_idx += 1
+        secondary_header.subservice = header_start[current_idx]
+        current_idx += 1
+        if pus_version == PusVersion.PUS_A:
+            secondary_header.message_counter = header_start[current_idx]
+            current_idx += 1
+        else:
+            secondary_header.message_counter = (
+                header_start[current_idx] << 8 | header_start[current_idx + 1]
+            )
+            current_idx += 2
+        if pus_version == PusVersion.PUS_C:
+            secondary_header.dest_id = (
+                header_start[current_idx] << 8 | header_start[current_idx + 1]
+            )
+            current_idx += 2
+        # If other time formats are supported in the future, this information can be used
+        #  to unpack the correct time code
+        time_code_id = read_p_field(header_start[current_idx])
+        if time_code_id:
+            pass
+        secondary_header.time = CdsShortTimestamp.unpack(
+            time_field=header_start[
+                current_idx : current_idx + PusTelemetry.PUS_TIMESTAMP_SIZE
+            ]
+        )
+        return secondary_header
+
+    def __repr__(self):
+        return (
+            f"{self.__class__.__name__}(service={self.service!r}, subservice={self.subservice!r}, "
+            f"time={self.time!r}, message_counter={self.message_counter!r}, "
+            f"dest_id={self.dest_id!r}, spacecraft_time_ref={self.spacecraft_time_ref!r}, "
+            f"pus_version={self.pus_version!r})"
+        )
+
+    @property
+    def header_size(self) -> int:
+        if self.pus_version == PusVersion.PUS_A:
+            return PusTelemetry.PUS_TIMESTAMP_SIZE + 4
+        else:
+            return PusTelemetry.PUS_TIMESTAMP_SIZE + 7
+
+
 class PusTelemetry:
     """Generic PUS telemetry class representation.
     It is instantiated by passing the raw pus telemetry packet (bytearray) to the constructor.
@@ -40,10 +222,10 @@ class PusTelemetry:
         self,
         service: int,
         subservice: int,
-        time: CdsShortTimestamp = None,
-        ssc: int = 0,
+        time: Optional[CdsShortTimestamp] = None,
         source_data: bytearray = bytearray([]),
-        apid: int = -1,
+        ssc: int = 0,
+        apid: int = FETCH_GLOBAL_APID,
         message_counter: int = 0,
         space_time_ref: int = 0b0000,
         destination_id: int = 0,
@@ -51,7 +233,7 @@ class PusTelemetry:
         pus_version: PusVersion = PusVersion.GLOBAL_CONFIG,
         secondary_header_flag: bool = True,
     ):
-        if apid == -1:
+        if apid == FETCH_GLOBAL_APID:
             apid = get_default_tm_apid()
         if pus_version == PusVersion.GLOBAL_CONFIG:
             pus_version = get_pus_tm_version()
@@ -64,20 +246,20 @@ class PusTelemetry:
         data_length = self.get_source_data_length(
             timestamp_len=PusTelemetry.PUS_TIMESTAMP_SIZE, pus_version=pus_version
         )
-        self.space_packet_header = SpacePacketHeader(
+        self.sp_header = SpacePacketHeader(
             apid=apid,
             packet_type=packet_type,
-            secondary_header_flag=secondary_header_flag,
-            packet_version=packet_version,
-            data_length=data_length,
-            source_sequence_count=ssc,
+            sec_header_flag=secondary_header_flag,
+            ccsds_version=packet_version,
+            data_len=data_length,
+            seq_count=ssc,
         )
-        self.secondary_packet_header = PusTmSecondaryHeader(
+        self.pus_tm_sec_header = PusTmSecondaryHeader(
             pus_version=pus_version,
-            service_id=service,
-            subservice_id=subservice,
+            service=service,
+            subservice=subservice,
             message_counter=message_counter,
-            destination_id=destination_id,
+            dest_id=destination_id,
             spacecraft_time_ref=space_time_ref,
             time=time,
         )
@@ -89,23 +271,25 @@ class PusTelemetry:
         cls, pus_version: PusVersion = PusVersion.GLOBAL_CONFIG
     ) -> PusTelemetry:
         return PusTelemetry(
-            service=0, subservice=0, time=CdsShortTimestamp.init_from_current_time()
+            service=0,
+            subservice=0,
+            time=CdsShortTimestamp.init_from_current_time(),
+            pus_version=pus_version,
         )
 
     def pack(self) -> bytearray:
         """Serializes the PUS telemetry into a raw packet."""
         tm_packet_raw = bytearray()
         # PUS Header
-        tm_packet_raw.extend(self.space_packet_header.pack())
+        tm_packet_raw.extend(self.sp_header.pack())
         # PUS Source Data Field
-        tm_packet_raw.extend(self.secondary_packet_header.pack())
+        tm_packet_raw.extend(self.pus_tm_sec_header.pack())
         # Source Data
         tm_packet_raw.extend(self._source_data)
         # CRC16-CCITT checksum
         crc_func = mkPredefinedCrcFun(crc_name="crc-ccitt-false")
         self._crc16 = crc_func(tm_packet_raw)
-        tm_packet_raw.append((self._crc16 & 0xFF00) >> 8)
-        tm_packet_raw.append(self._crc16 & 0xFF)
+        tm_packet_raw.extend(struct.pack("!H", self._crc16))
         return tm_packet_raw
 
     @classmethod
@@ -129,11 +313,9 @@ class PusTelemetry:
             raise ValueError
         pus_tm = cls.__empty(pus_version=pus_version)
         pus_tm._valid = False
-        pus_tm.space_packet_header = SpacePacketHeader.unpack(
-            space_packet_raw=raw_telemetry
-        )
+        pus_tm.sp_header = SpacePacketHeader.unpack(space_packet_raw=raw_telemetry)
         expected_packet_len = get_total_space_packet_len_from_len_field(
-            pus_tm.space_packet_header.data_length
+            pus_tm.sp_header.data_len
         )
         if expected_packet_len > len(raw_telemetry):
             logger = get_console_logger()
@@ -142,13 +324,13 @@ class PusTelemetry:
                 f"shorter than specified packet length in PUS header {expected_packet_len}"
             )
             raise ValueError
-        pus_tm.secondary_packet_header = PusTmSecondaryHeader.unpack(
+        pus_tm.pus_tm_sec_header = PusTmSecondaryHeader.unpack(
             header_start=raw_telemetry[SPACE_PACKET_HEADER_SIZE:],
             pus_version=pus_version,
         )
         if (
             expected_packet_len
-            < pus_tm.secondary_packet_header.header_size + SPACE_PACKET_HEADER_SIZE
+            < pus_tm.pus_tm_sec_header.header_size + SPACE_PACKET_HEADER_SIZE
         ):
             logger = get_console_logger()
             logger.warning("Passed packet too short!")
@@ -157,32 +339,50 @@ class PusTelemetry:
             logger = get_console_logger()
             logger.warning(
                 f"PusTelemetry: Packet length field "
-                f"{pus_tm.space_packet_header.data_length} might be invalid!"
+                f"{pus_tm.sp_header.data_len} might be invalid!"
             )
             logger.warning(f"Packet size from size field: {pus_tm.packet_len}")
             logger.warning(f"Length of raw telemetry: {len(raw_telemetry)}")
         pus_tm._source_data = raw_telemetry[
-            pus_tm.secondary_packet_header.header_size + SPACE_PACKET_HEADER_SIZE : -2
+            pus_tm.pus_tm_sec_header.header_size + SPACE_PACKET_HEADER_SIZE : -2
         ]
-        pus_tm._crc = (
-            raw_telemetry[expected_packet_len - 2] << 8
-            | raw_telemetry[expected_packet_len - 1]
-        )
+        pus_tm._crc = struct.unpack(
+            "!H", raw_telemetry[expected_packet_len - 2 : expected_packet_len]
+        )[0]
         pus_tm.__perform_crc_check(raw_telemetry=raw_telemetry[:expected_packet_len])
         return pus_tm
 
+    @classmethod
+    def from_composite_fields(
+        cls, sph: SpacePacketHeader, sec_header: PusTmSecondaryHeader, tm_data: bytes
+    ) -> PusTelemetry:
+        pus_tm = cls.__empty()
+        if sph.packet_type == PacketTypes.TC:
+            raise ValueError(
+                f"Invalid Packet Type {sph.packet_type} in CCSDS primary header"
+            )
+        pus_tm.sp_header = sph
+        pus_tm.pus_tm_sec_header = sec_header
+        pus_tm._source_data = tm_data
+        return pus_tm
+
+    def to_space_packet(self):
+        user_data = bytearray(self._source_data)
+        user_data.extend(struct.pack("!H", self.crc16))
+        return SpacePacket(self.sp_header, self.pus_tm_sec_header.pack(), user_data)
+
     def __str__(self):
         return (
-            f"PUS TM[{self.secondary_packet_header.service_id},"
-            f"{self.secondary_packet_header.subservice_id}], APID {self.apid:#05x}, MSG Counter "
-            f"{self.secondary_packet_header.message_counter}, Size {self.packet_len}"
+            f"PUS TM[{self.pus_tm_sec_header.service},"
+            f"{self.pus_tm_sec_header.subservice}], APID {self.apid:#05x}, MSG Counter "
+            f"{self.pus_tm_sec_header.message_counter}, Size {self.packet_len}"
         )
 
     def __repr__(self):
         return (
-            f"{self.__class__.__name__}(service={self.secondary_packet_header.service_id!r}, "
-            f"subservice={self.secondary_packet_header.subservice_id!r}, "
-            f"apid={self.apid!r}, ssc={self.ssc!r})"
+            f"{self.__class__.__name__}.from_composite_fields({self.__class__.__name__}"
+            f"(sph={self.sp_header!r}, sec_header={self.pus_tm_sec_header!r}, "
+            f"tm_data={self.tm_data!r}"
         )
 
     @property
@@ -190,14 +390,14 @@ class PusTelemetry:
         """Get the service type ID
         :return: Service ID
         """
-        return self.secondary_packet_header.service_id
+        return self.pus_tm_sec_header.service
 
     @property
     def subservice(self) -> int:
         """Get the subservice type ID
         :return: Subservice ID
         """
-        return self.secondary_packet_header.subservice_id
+        return self.pus_tm_sec_header.subservice
 
     @property
     def valid(self) -> bool:
@@ -211,8 +411,8 @@ class PusTelemetry:
         return self._source_data
 
     @property
-    def packet_id(self) -> int:
-        return self.space_packet_header.packet_id
+    def packet_id(self):
+        return self.sp_header.packet_id
 
     def __perform_crc_check(self, raw_telemetry: bytes):
         # CRC16-CCITT checksum
@@ -267,18 +467,18 @@ class PusTelemetry:
         The space packet data field is the full length of data field minus one without
         the space packet header.
         """
-        return self.space_packet_header.packet_len
+        return self.sp_header.packet_len
 
     @property
     def apid(self) -> int:
-        return self.space_packet_header.apid
+        return self.sp_header.apid
 
     @property
-    def ssc(self) -> int:
+    def seq_count(self) -> int:
         """Get the source sequence count
         :return: Source Sequence Count (see below, or PUS documentation)
         """
-        return self.space_packet_header.ssc
+        return self.sp_header.seq_count
 
     @property
     def crc16(self) -> int:
@@ -309,164 +509,3 @@ class PusTelemetry:
             data=self._source_data,
             length=len(self._source_data),
         )
-
-
-class PusTmSecondaryHeader:
-    """Unpacks the PUS telemetry packet secondary header.
-    Currently only supports CDS short timestamps"""
-
-    HEADER_SIZE_WITHOUT_TIME_PUS_A = 4
-    HEADER_SIZE_WITHOUT_TIME_PUS_C = 7
-
-    def __init__(
-        self,
-        service_id: int,
-        subservice_id: int,
-        time: CdsShortTimestamp,
-        message_counter: int,
-        destination_id: int = 0,
-        spacecraft_time_ref: int = 0,
-        pus_version: PusVersion = PusVersion.GLOBAL_CONFIG,
-    ):
-        """Create a PUS telemetry secondary header object.
-
-        :param pus_version: PUS version. ESA PUS is not supported
-        :param service_id:
-        :param subservice_id:
-        :param time: Time field
-        :param message_counter: 8 bit counter for PUS A, 16 bit counter for PUS C
-        :param destination_id: Destination ID if PUS C is used
-        :param spacecraft_time_ref: Space time reference if PUS C is used
-        """
-        self.pus_version = pus_version
-        if self.pus_version == PusVersion.GLOBAL_CONFIG:
-            self.pus_version = get_pus_tm_version()
-        if (
-            self.pus_version != PusVersion.PUS_A
-            and self.pus_version != PusVersion.PUS_C
-        ):
-            raise ValueError
-        self.spacecraft_time_ref = spacecraft_time_ref
-        self.service_id = service_id
-        self.subservice_id = subservice_id
-        if (
-            self.pus_version == PusVersion.PUS_A and message_counter > pow(2, 8) - 1
-        ) or (
-            self.pus_version == PusVersion.PUS_C and message_counter > pow(2, 16) - 1
-        ):
-            raise ValueError
-        self.message_counter = message_counter
-        self.destination_id = destination_id
-        self.time = time
-
-    @classmethod
-    def __empty(cls) -> PusTmSecondaryHeader:
-        return PusTmSecondaryHeader(
-            pus_version=PusVersion.PUS_C,
-            service_id=-1,
-            subservice_id=-1,
-            time=CdsShortTimestamp.init_from_current_time(),
-            message_counter=0,
-        )
-
-    def pack(self) -> bytearray:
-        secondary_header = bytearray()
-        if self.pus_version == PusVersion.PUS_A:
-            secondary_header.append((self.pus_version & 0x07) << 4)
-        elif self.pus_version == PusVersion.PUS_C:
-            secondary_header.append(self.pus_version << 4 | self.spacecraft_time_ref)
-        secondary_header.append(self.service_id)
-        secondary_header.append(self.subservice_id)
-        if self.pus_version == PusVersion.PUS_A:
-            secondary_header.append(self.message_counter)
-        elif self.pus_version == PusVersion.PUS_C:
-            secondary_header.append((self.message_counter & 0xFF00) >> 8)
-            secondary_header.append(self.message_counter & 0xFF)
-            secondary_header.append((self.destination_id & 0xFF00) >> 8)
-            secondary_header.append(self.destination_id & 0xFF)
-        secondary_header.extend(self.time.pack())
-        return secondary_header
-
-    @classmethod
-    def unpack(
-        cls, header_start: bytes, pus_version: PusVersion
-    ) -> PusTmSecondaryHeader:
-        """Unpack the PUS TM secondary header from the raw packet starting at the header index.
-        The user still needs to specify the PUS version because the version field is parsed
-        differently depending on the PUS version.
-
-        :param header_start:
-        :param pus_version:
-        :raises ValueError: bytearray too short or PUS version missmatch.
-        :return:
-        """
-        if len(header_start) < cls.HEADER_SIZE_WITHOUT_TIME_PUS_A:
-            logger = get_console_logger()
-            logger.warning("Passed bytearray too short")
-            raise ValueError
-        if pus_version == PusVersion.GLOBAL_CONFIG:
-            pus_version = get_pus_tm_version()
-        secondary_header = cls.__empty()
-        current_idx = 0
-        if pus_version == PusVersion.PUS_A:
-            secondary_header.pus_version = (header_start[current_idx] & 0x70) >> 4
-            if secondary_header.pus_version != PusVersion.PUS_A:
-                logger = get_console_logger()
-                logger.warning(
-                    f"PUS version field value {secondary_header.pus_version} "
-                    f"found where PUS A {pus_version} was expected!"
-                )
-                raise ValueError
-        elif pus_version == PusVersion.PUS_C:
-            secondary_header.pus_version = (header_start[current_idx] & 0xF0) >> 4
-            if secondary_header.pus_version != PusVersion.PUS_C:
-                logger = get_console_logger()
-                logger.warning(
-                    f"PUS version field value {secondary_header.pus_version} "
-                    f"found where PUS C {pus_version} was expected!"
-                )
-                raise ValueError
-            secondary_header.spacecraft_time_ref = header_start[current_idx] & 0x0F
-        if len(header_start) < secondary_header.header_size:
-            logger = get_console_logger()
-            logger.warning(
-                f"Invalid PUS data field header size, "
-                f"less than expected {secondary_header.header_size} bytes"
-            )
-            raise ValueError
-        current_idx += 1
-        secondary_header.service_id = header_start[current_idx]
-        current_idx += 1
-        secondary_header.subservice_id = header_start[current_idx]
-        current_idx += 1
-        if pus_version == PusVersion.PUS_A:
-            secondary_header.message_counter = header_start[current_idx]
-            current_idx += 1
-        else:
-            secondary_header.message_counter = (
-                header_start[current_idx] << 8 | header_start[current_idx + 1]
-            )
-            current_idx += 2
-        if pus_version == PusVersion.PUS_C:
-            secondary_header.destination_id = (
-                header_start[current_idx] << 8 | header_start[current_idx + 1]
-            )
-            current_idx += 2
-        # If other time formats are supported in the future, this information can be used
-        #  to unpack the correct time code
-        time_code_id = read_p_field(header_start[current_idx])
-        if time_code_id:
-            pass
-        secondary_header.time = CdsShortTimestamp.unpack(
-            time_field=header_start[
-                current_idx : current_idx + PusTelemetry.PUS_TIMESTAMP_SIZE
-            ]
-        )
-        return secondary_header
-
-    @property
-    def header_size(self) -> int:
-        if self.pus_version == PusVersion.PUS_A:
-            return PusTelemetry.PUS_TIMESTAMP_SIZE + 4
-        else:
-            return PusTelemetry.PUS_TIMESTAMP_SIZE + 7
